/*************************************************************************************
 											CoAN-008 application
								Datalogger over FAT16 over miniSD card
**************************************************************************************/
#if !__SEPARATE_INST_DATA__
#memmap xmem
#endif

#class auto
#use RCM43xx.LIB

//my libraries
#use flags.lib
#use logs.lib

//led defines and macros
#define DS2_BIT 	2
#define ALARMON   	BitWrPortI ( PBDR, &PBDRShadow, 0,DS2_BIT )
#define ALARMOFF  	BitWrPortI ( PBDR, &PBDRShadow, 1,DS2_BIT )

#define DS3_BIT	3
#define STATUSON   	BitWrPortI ( PBDR, &PBDRShadow, 0,DS3_BIT )
#define STATUSOFF  	BitWrPortI ( PBDR, &PBDRShadow, 1,DS3_BIT )

//sw defines
#define S2_BIT 4
#define S3_BIT 5

//Interval sample for log process
#define SAMPLEINTERVAL	300
//variables for measures simulation
static float ad_inputs[4];
static long qd_reading;

/****************************************
*			Printer functions					 *
*****************************************/
void print_head(){
	puts("\n  _________ _________ _________ _________ ____________ ________ ________\n");
	puts(" |   An0   |   An1   |   An2   |   An3   | QD reading | Log in | SeqNum |\n");
	puts(" |_________|_________|_________|_________|____________|________| _______|\n");
	puts(" |         |         |         |         |            |        |        |\n");
}
extern protected unsigned long SeqNumber;
void print_body()
{
   printf(" | %.2f | %.2f | %.2f | %.2f | %10ld |   %c    | 0x%04X |\r",
   			ad_inputs[0],ad_inputs[1],ad_inputs[2],ad_inputs[3], qd_reading,
  				(FLGLOGSTARTED)?'S':'N', SeqNumber);
}

main()
{
	int channel;
   // restore any protected variables
   _sysIsSoftReset();
   // board functions initialization
  	brdInit();
   //Flags initialization
   flags_init();

   print_head();

	while(1){
      /***********************************************************************
       * SIULATE SAMPLES
       **********************************************************************/
      costate
      {  // simulate AD samples for each channel
			for(channel = 0; channel <= 3; channel++)
				ad_inputs[channel] = (rand()-0.5)/10 + 5567;

		  //take QD simulated counter value
			qd_reading += (long)(10*(rand()- 0.5))%10;
			waitfor( DelayMs( 250 ) );
		}
      /***********************************************************************
       * DATALOG PROCESS
       **********************************************************************/
      /**********************************
       * start / stop logging conditions
       *********************************/
       costate
       {
       		if (!BitRdPortI(PBDR, S2_BIT) )		//wait for switch S2 press
            {
            	if( FLGLOGSTARTED ){
             	  puts("\n\nPreparando detencion de logs\n");
                 FLGLOGSTARTED_RST;
             	}
             	else{
              	  puts("\n\nPreparando inicio de logs\n");
                 FLGLOGSTARTED_SET;
             	}
            }
            waitfor(DelayMs(300));
       }
      /**********************************
      *  start / stop logging actions
      *********************************/
       costate
       { //Logs off -> off
         if( !FLGCHANGED(LOGSTARTED_bit) && !FLGLOGSTARTED )
         	abort;

         //Logs on -> on
         if( !FLGCHANGED(LOGSTARTED_bit) && FLGLOGSTARTED )
         {
            waitfor(DelayMs(SAMPLEINTERVAL));

            if( !(FLGMEMNOINSERTED || FLGMEMFULL)  )
            {
            	switch( log_insert() ) //New log
             	{
             		case LOGMEMAVAILABLE:
               		break;
               	case LOGMEMNOINSERTED:
               		FLGMEMNOINSERTED_SET;
               		break;
               	case LOGMEMFULL:
	               	FLGMEMFULL_SET;
               		break;
               	default:
               		break;
               }
            }
	         else
   	      {  //If mem_no_error -> mem_error)
               if ( (FLGCHANGED(MEMNOINSERTED_bit) && FLGMEMNOINSERTED) ||
                    (FLGCHANGED(MEMFULL_bit) && FLGMEMFULL)  )
               {  //unmount FAT
	              	if (log_filesystem_close() == SUCCESS )
		               FLGFILESYSTEMSTARTED_RST;
                  //update flags after changed detected
                  FLGLASTUPD( MEMNOINSERTED_bit );
                  FLGLASTUPD( MEMFULL_bit );
               }
               //THE USER MUST SOLVE THE MEMORY PROBLEM, AND MUST RESTART THE LOG PROCESS.
            }
            abort;
      	}

         //Logs off -> on
       	if( FLGCHANGED(LOGSTARTED_bit) && FLGLOGSTARTED   )
         {  //Mount FAT if is unmounted
         	if ( !FLGFILESYSTEMSTARTED )
            {  //if come from an exception, dont format the partition!!!.
         		if ( log_filesystem_init( !FLGLOGGINGCURRENTLY ) == SUCCESS ){
            		FLGFILESYSTEMSTARTED_SET;
                  print_head();    //for stdio print
               }
            }

           	switch( log_mem_available() ){
            	case LOGMEMAVAILABLE:
	            	FLGLASTUPD( LOGSTARTED_bit );//flag changed->update
                  FLGMEMNOINSERTED_RST;
                  FLGMEMFULL_RST;
                  cfgSaveLoggingCurrently(1);//Set persistent indication of loggin state
               	break;                     //preventing formatting after power cuts
               case LOGMEMNOINSERTED:
	            	FLGMEMNOINSERTED_SET;
               	break;
               case LOGMEMFULL:
               	FLGMEMNOINSERTED_RST;//if mem full, then memory is inserted now
               	FLGMEMFULL_SET;
                  FLGLASTUPD( LOGSTARTED_bit );//flag changed->update
                  break;
               default:
               	break;
            }

            abort;
         }
         //Logs  on -> off
         if ( FLGCHANGED(LOGSTARTED_bit) && !FLGLOGSTARTED )
         {  //desmonto la FAT;
         	if ( log_filesystem_close() == SUCCESS ){
            	FLGFILESYSTEMSTARTED_RST;
               print_head();
            }
           	FLGLASTUPD( LOGSTARTED_bit );//flag changed->update
            cfgSaveLoggingCurrently(0);  //Reset persistent indication of loggin state
            abort;
         }

       }
      /***************************
      	ALARM LED CONTROL
      ****************************/
		costate
      {
      	if( FLGMEMFULL || FLGMEMNOINSERTED){
         	ALARMON;
            abort;
         }
         else{
	         ALARMOFF;
            abort;
         }
      }
      /***************************
      	STATUS LED CONTROL,
         although, only blinking the led
      ****************************/
		costate
      {
				STATUSON;
            waitfor(DelayMs(750));
            STATUSOFF;
            waitfor(DelayMs(750));
            print_body();
      }

	}
}


